/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#include <src/investigation/analyseur/analyseur.h>
#define YYLTYPE Investigation::Global::Source::Position
#include <src/investigation/analyseur/includes.h>
#include <src/investigation/analyseur/analyseur.syn.h>

#include <sys/stat.h>
#include <unistd.h>
#define FICHIER_STDIN "-"
#include <config.h>

using namespace Investigation::Analyseur;

int parseurparse(Investigation::Donnees::SiteSP&,Investigation::Donnees::DebugConditionsSP&);
int parseurlex_destroy();
extern FILE* parseurin;
extern Investigation::Global::Source::Position parseurlloc;

Analyseur::~Analyseur()
{
	parseurlex_destroy();
}

Investigation::Donnees::SiteSP Analyseur::analyse(const std::string& fichier)
{
	Analyseur analyseur;
	parseurin = Analyseur::ouvrir(fichier,parseurlloc);
	Investigation::Donnees::SiteSP site;
	Investigation::Donnees::DebugConditionsSP debug_conditions;
	parseurparse(site,debug_conditions);
	::fclose(parseurin);
	return site;
}

bool Analyseur::teste_fichier(std::string fichier)
{
	if(::access(fichier.c_str(),R_OK))
		return false;
	struct stat buf;
	if(::stat(fichier.c_str(),&buf))
		return false;

	if(S_ISDIR(buf.st_mode))
		return false;
	return true;
}

FILE *Analyseur::ouvrir(std::string fichier, Investigation::Global::Source::Position& position)
{
	if((fichier==FICHIER_STDIN) or (fichier.empty()))
	{
                position = Investigation::Global::Source::Position(FICHIER_STDIN);
		return stdin;
	}

	if(not teste_fichier(fichier)) 
		throw Investigation::Global::Exception::PasDeFichierSource(fichier);

	FILE *fic = ::fopen(fichier.c_str(),"r");
        position = Investigation::Global::Source::Position(fichier);
	return fic;
}
