/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#ifndef __INVESTIGATION_DONNEES_PAGE__
#define __INVESTIGATION_DONNEES_PAGE__

#include <vector>
#include <src/global/global.h>
namespace Investigation { namespace Donnees { DECL_SHARED_PTR(Page); DECL_SHARED_PTR(ContenuPage); typedef std::vector<ContenuPageSP> SequenceContenuPage; } }
#include <src/investigation/donnees/instruction.h>
#include <src/investigation/donnees/html.h>
#include <src/investigation/donnees/requete.h>
#include <src/investigation/donnees/liens.h>

namespace Investigation
{
	namespace Donnees
	{
		class ContenuPage
		{
			public:
				virtual ~ContenuPage() {};
				virtual Html::SequenceContenuDiv operator() (const std::string& methode, const Investigation::Donnees::Requete& requete, Investigation::Donnees::Liens& liens) const = 0;
				virtual void liens_valides(const std::set<std::string>& identifiants) const = 0;
				virtual void propage_infos(const std::string& page) = 0;

		};

		typedef std::vector<PageSP> SequencePages;
		class Page : public Instruction
		{
			public:
				Page(const bool brouillon, const std::vector<std::string>& identifiants, const SequenceContenuPage& contenu, const Investigation::Global::Source::PositionSP& position)
				:Instruction(position), _brouillon(brouillon), _identifiants(identifiants.begin(),identifiants.end()), _contenu(contenu) { };
				Page(const bool brouillon, const std::vector<std::string>& identifiants, const std::string& titre, const SequenceContenuPage& contenu, const Investigation::Global::Source::PositionSP& position)
				:Instruction(position), _brouillon(brouillon), _identifiants(identifiants.begin(),identifiants.end()), _titre(titre), _contenu(contenu) { };
				virtual ~Page() {};
				virtual Investigation::Donnees::SequencePages operator() (const Investigation::Donnees::Requete& requete) const
				{
					Investigation::Donnees::SequencePages resultat;
					std::string motclef = requete;
					if(_identifiants.find(motclef)!=_identifiants.end())
					{
						PageSP page(new Page(*this));
						resultat.push_back(page);
					}
					return resultat;
				};
				Html::Div operator() (const std::string& methode, const Investigation::Donnees::Requete& requete, Investigation::Donnees::Liens& liens) const
				{
					Html::Div div_page(_brouillon?"investigation page draft":"investigation page");
					Html::Hr hr_page("investigation page");
					Html::H titre_page(2,"investigation page title");
					titre_page << Html::Texte(_titre);
					div_page << titre_page;
					for(auto it=_contenu.cbegin() ; it!=_contenu.cend() ; ++it)
					{
						div_page << (**it)(methode,requete,liens);
					}
					return div_page;
				}
				virtual std::set<std::string> identifiants() const
				{
					return _identifiants;
				};
				virtual void liens_valides(const std::set<std::string>& identifiants) const
				{
					for(auto it=_contenu.cbegin() ; it!=_contenu.cend() ; ++it)
					{
						(*it)->liens_valides(identifiants);
					}
				};
				virtual void propage_infos()
				{
					for(auto it=_contenu.cbegin() ; it!=_contenu.cend() ; ++it)
					{
						(*it)->propage_infos(_titre);
					}
				};
			private:
				bool _brouillon;
				std::set<std::string> _identifiants;
				std::string _titre;
				SequenceContenuPage _contenu;
		};
	}
}
#endif
