/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#ifndef __INVESTIGATION_DONNEES_QUESTION__
#define __INVESTIGATION_DONNEES_QUESTION__

#include <vector>
#include <src/global/global.h>
namespace Investigation { namespace Donnees { DECL_SHARED_PTR(Question); DECL_SHARED_PTR(Reponse); typedef std::vector<ReponseSP> SequenceReponses; } }
#include <src/investigation/donnees/page.h>

namespace Investigation
{
	namespace Donnees
	{
		class Reponse
		{
			public:
				virtual ~Reponse() {};
				virtual Html::SequenceLi operator() (const std::string& methode, const Investigation::Donnees::Requete& requete, Investigation::Donnees::Liens& liens) const = 0;
				virtual void liens_valides(const std::set<std::string>& identifiants) const = 0;
				virtual void propage_infos(const std::string& page, const std::string& question) = 0;
		};

		class Question : public ContenuPage
		{
			public:
				Question(const std::string& question, const SequenceReponses& reponses)
				:_question(question), _reponses(reponses) { };
				virtual ~Question() {};
				virtual Html::SequenceContenuDiv operator() (const std::string& methode, const Investigation::Donnees::Requete& requete, Investigation::Donnees::Liens& liens) const
				{
					Html::SequenceContenuDiv seq;
					Html::P question("investigation page question");
					question << Html::Texte(_question);
					seq.push_back(question);
					Html::Ul liste_reponses("investigation page response");
					for(auto it=_reponses.cbegin() ; it!=_reponses.cend() ; ++it)
					{
						liste_reponses << (**it)(methode,requete,liens);
					}
					seq.push_back(liste_reponses);
					return seq;
				}
				virtual void liens_valides(const std::set<std::string>& identifiants) const
				{
					for(auto it=_reponses.cbegin() ; it!=_reponses.cend() ; ++it)
					{
						(*it)->liens_valides(identifiants);
					}
				};
				virtual void propage_infos(const std::string& page)
				{
					for(auto it=_reponses.cbegin() ; it!=_reponses.cend() ; ++it)
					{
						(*it)->propage_infos(page,_question);
					}
				};
			private:
				std::string _question;
				SequenceReponses _reponses;
		};

	}
}
#endif
