/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#ifndef __INVESTIGATION_DONNEES_REPONSECHOIX__
#define __INVESTIGATION_DONNEES_REPONSECHOIX__

#include <vector>
#include <src/global/global.h>
namespace Investigation { namespace Donnees { DECL_SHARED_PTR(ReponseChoix); } }
#include <src/investigation/donnees/question.h>

namespace Investigation
{
	namespace Donnees
	{

		class LienInvalide : public Investigation::Global::Exception::Compilation
		{
			public:
				LienInvalide(const Investigation::Global::Source::PositionSP& position, const std::string& identifiant)
				:Compilation(position,std::string("Link to pages ")+identifiant+" invalid") {};
		};

		class ReponseChoix : public Reponse
		{
			public:
				ReponseChoix(const std::string& identifiant, const std::string& intitule, const Investigation::Global::Source::PositionSP& position)
				:_identifiant(identifiant), _intitule(intitule), _position(position) { };
				virtual ~ReponseChoix() { };
				virtual Html::SequenceLi operator() (const std::string& methode, const Investigation::Donnees::Requete& requete, Investigation::Donnees::Liens& liens) const
				{
					Html::Li item_reponse("investigation page response");
					Requete nouvelle_requete = (requete << Requete::MotClef(_identifiant,_intitule,_question,_page));
					std::string lien = liens[nouvelle_requete];
					Html::A lien_reponse(methode+lien,"investigation page response");
					lien_reponse << Html::Texte(_intitule);
					item_reponse << lien_reponse;
					Html::SequenceLi seq;
					seq.push_back(item_reponse);
					return seq;
				};
				virtual void liens_valides(const std::set<std::string>& identifiants) const
				{
					if(identifiants.find(_identifiant)==identifiants.end())
					{
						throw LienInvalide(_position,_identifiant);
					}
				};
				virtual void propage_infos(const std::string& page, const std::string& question)
				{
					_question=question;
					_page=page;
				};
			private:
				std::string _identifiant;
				std::string _intitule;
				std::string _question;
				std::string _page;
				Investigation::Global::Source::PositionSP _position;
		};
	}
}
#endif
