/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#ifndef __INVESTIGATION_DONNEES_REQUETE__
#define __INVESTIGATION_DONNEES_REQUETE__

#include <string>
#include <map>
#include <vector>
#include <src/global/global.h>
namespace Investigation { namespace Donnees { DECL_SHARED_PTR(Requete); } }

namespace Investigation
{
	namespace Donnees
	{
		class Liens;
		class ConditionIdentifiant;
		template<typename Patron>
		class ConditionExpressionRationnelle;
		class DebugConditions;
		class Requete
		{
			friend class ConditionIdentifiant;
			template<typename Patron> friend class ConditionExpressionRationnelle;
			friend class DebugConditions;
			public:
				struct MotClef
				{
					friend class Requete;
					MotClef(const std::string& motclef, const std::string& reponse, const std::string& question, const std::string& page)
					:_motclef(motclef), _reponse(reponse), _question(question), _page(page) {};
					operator std::string() const
					{
						std::ostringstream oss;
						if(not _page.empty())
						{
							oss << "On page [" << _page << "]. ";
						}
						if(not _question.empty())
						{
							oss << "For question [" << _question << "]. ";
						}
						if(not _reponse.empty())
						{
							oss << "Answer is [" << _reponse << "]. ";
						}
						return oss.str();
					}
					bool operator==(const MotClef& motclef) const
					{
						return (_motclef==motclef._motclef) and (_reponse==motclef._reponse) and (_question==motclef._question) and (_page==motclef._page);
					}
					private:
						std::string _motclef;
						std::string _reponse;
						std::string _question;
						std::string _page;
						explicit MotClef(const std::string& motclef)
						:_motclef(motclef) {};
				};
				Requete() = default;
				Requete operator<<(const MotClef& motclef) const
				{
					Requete requete(*this);
					requete._liste.push_back(motclef);
					auto itmap = requete._occurences.find(motclef._motclef);
					if(itmap==requete._occurences.end())
					{
						requete._occurences[motclef._motclef]=1;
					}
					else
					{
						++requete._occurences[motclef._motclef];
					}
					return requete;
				}
				operator std::string () const
				{
					if(_liste.empty())
						throw;
					return _liste.back()._motclef;
				}
				operator std::vector<std::string> () const
				{
					std::vector<std::string> resultat;
					for(auto it=_liste.cbegin() ; it!=_liste.cend() ; ++it)
					{
						resultat.push_back(it->_motclef);
					}
					return resultat;
				}
				operator bool () const
				{
					return not _liste.empty();
				}
				bool operator==(const Requete& requete) const
				{
					if(_liste.size()!=requete._liste.size())
						return false;
					for(auto it=_liste.cbegin(), itr=requete._liste.cbegin() ; it!=_liste.cend() ; ++it,++itr)
					{
						if(not ((*it)==(*itr)))
							return false;
					}
					return true;
				}
				std::vector<std::pair<std::string, MotClef> > historique(Investigation::Donnees::Liens& liens) const;
				template<typename oStream>
				friend oStream& operator<<(oStream& os, const Requete& requete)
				{
					std::vector<std::string> mot_clefs = requete;
					for(auto it=mot_clefs.cbegin() ; it!=mot_clefs.cend() ; ++it)
					{
						os << " " << (*it);
					}
					return os;
				}
			private:
				std::vector<MotClef> _liste;
				std::map<std::string,size_t> _occurences;
				static Requete requete(const std::vector<std::string>& mot_clefs)
				{ 
					Requete requete;
					for(auto it=mot_clefs.cbegin() ; it!=mot_clefs.cend() ; ++it)
					{
						requete = (requete << MotClef(*it));
					}
					return requete;
				};
		};

	}
}
#endif
