/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#ifndef __INVESTIGATION_DONNEES_SELECTION__
#define __INVESTIGATION_DONNEES_SELECTION__

#include <vector>
#include <src/global/global.h>
namespace Investigation { namespace Donnees { } }
#include <src/investigation/donnees/condition.h>

namespace Investigation
{
	namespace Donnees
	{
		template<typename Contenu>
		class Selection
		{
			public:
				struct Clause
				{
					friend class Selection<Contenu>;
					Clause(const ConditionSP& condition, const std::vector<SHARED_PTR(Contenu)>& contenu)
					:_condition(condition), _contenu(contenu) {};
					private:
						ConditionSP _condition;
						std::vector<SHARED_PTR(Contenu)> _contenu;
						
				};
				typedef std::vector<Clause> SequenceClauses;
				Selection(const SequenceClauses& clauses, const std::vector<SHARED_PTR(Contenu)>& clause_par_defaut)
				:_clauses(clauses), _clause_par_defaut(clause_par_defaut) { };
				virtual ~Selection() {};
				Selection<Contenu>& operator<<(const Clause& clause)
				{
					_clauses.push_back(clause);
					return *this;
				}
				Selection<Contenu>& operator<<(const SHARED_PTR(Contenu)& clause_par_defaut)
				{
					_clause_par_defaut = clause_par_defaut;
					return *this;
				}
				std::vector<SHARED_PTR(Contenu)> operator() (const Investigation::Donnees::Requete& requete) const
				{
					for(auto it=_clauses.cbegin() ; it!=_clauses.cend() ; ++it)
					{
						if((*(it->_condition))(requete))
						{
							return it->_contenu;
						}
					}
					return _clause_par_defaut;
				};
				operator std::vector<SHARED_PTR(Contenu)> () const
				{
					std::vector<SHARED_PTR(Contenu)> resultat;
					for(auto it=_clauses.cbegin() ; it!=_clauses.cend() ; ++it)
					{
						resultat.insert(resultat.end(),it->_contenu.cbegin(),it->_contenu.cend());
					}
					resultat.insert(resultat.end(),_clause_par_defaut.cbegin(),_clause_par_defaut.cend());
					return resultat;
				};
			private:
				SequenceClauses _clauses;
				std::vector<SHARED_PTR(Contenu)> _clause_par_defaut;
		};

	}
}
#endif
