/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#ifndef __INVESTIGATION_DONNEES_SELECTIONPAGE__
#define __INVESTIGATION_DONNEES_SELECTIONPAGE__

#include <vector>
#include <src/global/global.h>
namespace Investigation { namespace Donnees { DECL_SHARED_PTR(SelectionPage); } }
#include <src/investigation/donnees/instruction.h>
#include <src/investigation/donnees/selection.h>

namespace Investigation
{
	namespace Donnees
	{

		class SelectionPage : public Instruction, public Selection<Instruction>
		{
			public:
				explicit SelectionPage(const typename Selection<Instruction>::SequenceClauses& clauses, const SequenceInstructions& clause_par_defaut, const Investigation::Global::Source::PositionSP& position)
				:Instruction(position), Selection(clauses,clause_par_defaut) { };
				virtual ~SelectionPage() {};
				virtual Investigation::Donnees::SequencePages operator() (const Investigation::Donnees::Requete& requete) const
				{
					Investigation::Donnees::SequencePages resultat;
					SequenceInstructions clause = (static_cast<Selection<Instruction> >(*this))(requete);
					for(auto it=clause.cbegin() ; it!=clause.cend() ; ++it)
					{
						Investigation::Donnees::SequencePages sous_resultat = (**it)(requete);
						resultat.insert(resultat.end(),sous_resultat.begin(),sous_resultat.end());
					}
					return resultat;
				}
				virtual std::set<std::string> identifiants() const
				{
					std::set<std::string> resultat;
					std::vector<InstructionSP> instructions = (*this);
					for(auto it=instructions.cbegin() ; it!=instructions.cend() ; ++it)
					{
						std::set<std::string> sous_resultat = (*it)->identifiants();
						resultat.insert(sous_resultat.cbegin(),sous_resultat.cend());
					}
					return resultat;
				};
				virtual void liens_valides(const std::set<std::string>& identifiants) const
				{
					std::vector<InstructionSP> instructions = (*this);
					for(auto it=instructions.cbegin() ; it!=instructions.cend() ; ++it)
					{
						(*it)->liens_valides(identifiants);
					}
					
				};
				virtual void propage_infos()
				{
					std::vector<InstructionSP> instructions = (*this);
					for(auto it=instructions.cbegin() ; it!=instructions.cend() ; ++it)
					{
						(*it)->propage_infos();
					}
				};
		};
	}
}
#endif
