/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#ifndef __INVESTIGATION_DONNEES_SELECTIONREPONSE__
#define __INVESTIGATION_DONNEES_SELECTIONREPONSE__

#include <vector>
#include <src/global/global.h>
namespace Investigation { namespace Donnees { DECL_SHARED_PTR(SelectionReponse); } }
#include <src/investigation/donnees/question.h>
#include <src/investigation/donnees/selection.h>

namespace Investigation
{
	namespace Donnees
	{
		class SelectionReponse : public Reponse, public Selection<Reponse>
		{
			public:
				SelectionReponse(const typename Selection<Reponse>::SequenceClauses& clauses, const SequenceReponses& clause_par_defaut)
				:Selection(clauses,clause_par_defaut) {};
				virtual ~SelectionReponse() {};
				virtual Html::SequenceLi operator() (const std::string& methode, const Investigation::Donnees::Requete& requete, Investigation::Donnees::Liens& liens) const
				{
					std::vector<ReponseSP> reponses = static_cast<Selection<Reponse> > (*this)(requete); 
					Html::SequenceLi seq;
					for(auto it=reponses.cbegin() ; it!=reponses.cend() ; ++it)
					{
						Html::SequenceLi sous_sequence = (**it)(methode,requete,liens);
						seq.insert(seq.end(),sous_sequence.cbegin(),sous_sequence.cend());
					}
					return seq;
				}
				virtual void liens_valides(const std::set<std::string>& identifiants) const
				{
					std::vector<ReponseSP> reponses = (*this);
					for(auto it=reponses.cbegin() ; it!=reponses.cend() ; ++it)
					{
						(*it)->liens_valides(identifiants);
					}
				}
				virtual void propage_infos(const std::string& page, const std::string& question)
				{
					std::vector<ReponseSP> reponses = (*this);
					for(auto it=reponses.cbegin() ; it!=reponses.cend() ; ++it)
					{
						(*it)->propage_infos(page,question);
					}
				}
		};
	}
}
#endif
