/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#ifndef __INVESTIGATION_DONNEES_SITE__
#define __INVESTIGATION_DONNEES_SITE__

#include <vector>
#include <set>
#include <string>
#include <src/global/global.h>
namespace Investigation { namespace Donnees { DECL_SHARED_PTR(Site); } }
#include <src/investigation/donnees/instruction.h>
#include <src/investigation/donnees/page.h>
#include <src/investigation/donnees/requete.h>
#include <src/investigation/donnees/html.h>
#include <src/investigation/donnees/liens.h>

namespace Investigation
{
	namespace Donnees
	{
		class Serveur;
		class Site
		{
			friend class Serveur;
			public:
				Site(const std::string& titre)
				:_titre(titre) {};
				void operator<<(const std::string& style)
				{
					_style=style;
				}
				void operator<<(const PageSP& page_de_demarrage)
				{
					_page_de_demarrage=page_de_demarrage;
				}
				void operator<<(const SequenceInstructions& instructions)
				{
					_instructions=instructions;
				}
				Investigation::Donnees::SequencePages operator()(const Investigation::Donnees::Requete& requete) const
				{
					Investigation::Donnees::SequencePages resultat;
					if(not requete)
					{
						resultat.push_back(_page_de_demarrage);
						return resultat;
					}
					resultat = (*_page_de_demarrage)(requete);
					for(auto it=_instructions.cbegin() ; it!=_instructions.cend() ; ++it)
					{
						Investigation::Donnees::SequencePages sous_resultat = (**it) (requete);
						resultat.insert(resultat.end(),sous_resultat.begin(),sous_resultat.end());
					}
					return resultat;
				}
				void integrite()
				{
					std::set<std::string> identifiants_valides = _page_de_demarrage->identifiants();
					for(auto it=_instructions.cbegin() ; it!=_instructions.cend() ; ++it)
					{
						auto sous_identifiants_valides = (*it)->identifiants();
						identifiants_valides.insert(sous_identifiants_valides.cbegin(),sous_identifiants_valides.cend());
					}
					_page_de_demarrage->liens_valides(identifiants_valides);
					for(auto it=_instructions.cbegin() ; it!=_instructions.cend() ; ++it)
					{
						(*it)->liens_valides(identifiants_valides);
						(*it)->propage_infos();
					}
					_page_de_demarrage->propage_infos();
				};
				Html::Html operator() (const Investigation::Donnees::SequencePages& pages, const std::string& methode, const Investigation::Donnees::Requete& requete, Investigation::Donnees::Liens& liens) const
				{
					Html::Html html(_titre,"/style.css");
					Html::H titre_resultat(1,"investigation result title");
					titre_resultat << Html::Texte("Investigation result");
					Html::Div div_resultat("investigation result");
					div_resultat << titre_resultat;
					for(auto it=pages.cbegin() ; it!=pages.cend() ; ++it)
					{
						div_resultat << (**it)(methode,requete,liens);
					}
					html << div_resultat;
					html << Html::Hr("investigation history");
					Html::Div div_historique("investigation history");
					Html::H titre_historique(1,"investigation history title");
					titre_historique << Html::Texte("Investigation history");
					div_historique << titre_historique;
					Html::Ol liste_historique("investigation history list");
					std::vector<std::pair<std::string, Investigation::Donnees::Requete::MotClef> > liens_historique = requete.historique(liens);
					for(auto it=liens_historique.cbegin() ; it!=liens_historique.cend() ; ++it)
					{
						Html::Li item_historique("investigation history item");
						Html::A lien_historique(methode+(it->first),"investigation history link");
						lien_historique << Html::Texte(it->second);
						item_historique << lien_historique;
						liste_historique << item_historique;
					}
					div_historique << liste_historique;
					html << div_historique;
					return html;
				};
			private:
				const std::string _titre;
				PageSP _page_de_demarrage;
				SequenceInstructions _instructions;
				std::string _style;
				
		};
	}
}
#endif
