/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#include <unistd.h>

#include <config.h>
#include <src/global/global.h>
#include <src/investigation/analyseur/analyseur.h>
#include <src/investigation/donnees/serveur.h>

#define HOTES_DEFAUT "localhost"
#define PORT_DEFAUT "http"

void aide(const std::string& programme)
{
	std::cout << programme << " [options] [file]" << std::endl
		<< std::endl
		<< "-H <allowed hosts>    : specify which hosts can request a page (default value: " << HOTES_DEFAUT << ")" << std::endl
		<< "-P <port to listen to>: specify the port used by clients to request pages (default value: " << PORT_DEFAUT << ")" << std::endl
		<< "-d                    : switch to debug mode - do not go to background, and display requests keywords" << std::endl
		<< "-v                    : display the program version" << std::endl
		<< "-h                    : display this help" << std::endl
		<< "" << std::endl
		<< "file                  : file used to build investigation site (- means standard input)" << std::endl;
}

int main(int argc, char *argv[])
{
	std::string hotes = HOTES_DEFAUT;
	std::string port = PORT_DEFAUT;
	bool tache_de_fond=true;
	int option;
	for(;;)
	{
		option=::getopt(argc,argv,"H:P:dhv");
		if(option==-1)
			break;
		switch(option)
		{
			case 'H':
				{
					if(hotes!=HOTES_DEFAUT)
						goto aide;
					hotes = optarg;
				}
				break;
			case 'P':
				{
					if(port!=PORT_DEFAUT)
						goto aide;
					port = optarg;
				}
				break;
			case 'd':
				{
					if(not tache_de_fond)
						goto aide;
					tache_de_fond=false;
				}
				break;
			case 'v':
				{
					std::cout << argv[0] << " version " << PACKAGE_VERSION "." << std::endl
						<< std::endl
						<< argv[0] << " Copyright (C) 2013  Julien Bruguier" << std::endl
						<< "This program comes with ABSOLUTELY NO WARRANTY. " << std::endl
						<< "This is free software, and you are welcome to redistribute it" << std::endl
						<< "under certain conditions. Please refer to the licence GPLv3" << std::endl
						<< "for details." << std::endl;
					return 0;
				}
				break;
			case 'h':
			default:
			aide:
				{
					aide(argv[0]);
					return 1;
				}
				break;
		}
	}
	if(optind>=argc)
	{
		aide(argv[0]);
		return 1;
	}
	std::string fichier(argv[optind]);
	try
	{
		Investigation::Donnees::SiteSP site = Investigation::Analyseur::Analyseur::analyse(fichier);
		if(not site)
			return 0;
		site->integrite();
		Investigation::Donnees::Serveur::serveur(tache_de_fond,hotes,port,site);
	}
	catch(Investigation::Global::Exception::Execution& e)
	{
		e.format(std::cerr);
		return 1;
	}
	catch(Investigation::Global::Exception::Compilation& e)
	{
		e.format(std::cerr);
		return 1;
	}
	return 0;
}
